/* compare.cc - compare static stuff
 * Rob Earhart
 * $Id: compare.cc,v 1.4 1998/02/18 21:16:51 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include "response.h"
#include "compare.h"

Compare *Compare::instance = NULL;
pthread_once_t Compare::instance_init = PTHREAD_ONCE_INIT;

void Compare::Initialize() {
  instance = new Compare();
}

Compare *Compare::Instance() {
  if (! instance)
    pthread_once(&instance_init, &Initialize);
  return instance;
}

Compare::Family *Compare::FindFamilyOfComparator(const String &name) {
  String::const_iterator start = name.begin(),
                         finish;
  if (*start == '+' || *start == '-')
    start++;
  for (finish = start;
       finish < name.end();
       ++finish)
    if (*finish == ';')
      break;
  {
    Grab g(family_lock);
    familymap_t::iterator family = families.find(String(start, finish));
    if (family == families.end())
      throw Response::BAD("Unknown comparator family");
    return (*family).second;
  }
}

Compare::ordering_t Compare::LookupOrdering(const String &name) {
  DEBUG("Looking for ordering " << name);
  Family *family = FindFamilyOfComparator(name);
  ordmap_t::iterator it = family->Orderings().find(name);
  if (it == family->Orderings().end())
    throw Response::BAD("Unknown ordering comparator");
  return (*it).second;
}

Compare::equality_t Compare::LookupEquality(const String &name) {
  DEBUG("Looking for equality " << name);
  Family *family = FindFamilyOfComparator(name);
  eqmap_t::iterator it = family->Equalities().find(name);
  if (it == family->Equalities().end())
    throw Response::BAD("Unknown equality comparator");
  return (*it).second;
}

Compare::match_t Compare::LookupMatch(const String &name) {
  DEBUG("Looking for match " << name);
  Family *family = FindFamilyOfComparator(name);
  matchmap_t::iterator it = family->Matches().find(name);
  if (it == family->Matches().end())
    throw Response::BAD("Unknown match comparator");
  return (*it).second;
}

