/* dispatch.h -*- C++ -*- command dispatch queue
 * Rob Earhart
 * $Id: dispatch.h,v 1.3 1998/02/18 21:17:37 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_DISPATCH_H
#define ACAP_DISPATCH_H

#include <deque.h>
#include <pthread.h>
#include "ptr.h"
#include "task.h"
#include "mutex.h"
#include "cond.h"

class Dispatch {
public:
  static Dispatch *Instance();	// Access the dispatch queue
  void Enqueue(Ptr<Task> &task); // Add a task to the queue
  void Loop() throw ();		// For threads that wish to donate
				// themselves to processing the dispatch queue
  // This mutex is used to ensure that commands which must acquire
  // threads in a specific order actually do so.
  Mutex &OrderingMutex() { return ordering_mutex; }
  void Shutdown();
private:
  static void Initialize();
  Dispatch() : workers(0), dying(false) {}
  static Dispatch *instance;
  static pthread_once_t instance_init;
  deque<Ptr<Task> > pending;
  Mutex pending_mutex;
  Cond pending_cond;
  Mutex ordering_mutex;
  Mutex loop_mutex;
  Cond shutdown_cond;
  unsigned workers;
  bool dying;
};

#endif // ACAP_DISPATCH_H
