/* language.h -*- C++ -*- language interface
 * Rob Earhart
 * $Id: language.h,v 1.3 1997/12/29 22:11:51 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_LANGUAGE_H
#define ACAP_LANGUAGE_H

#include <hash_map.h>
#include "msg.h"
#include "str.h"
#include "compare.h"

class Language {
public:
  typedef hash_map<String, const Language *,
                   hash<String>, equal_to<String> > langmap_t;
  static langmap_t &Langs() {
    if (! langs)
      langs = new langmap_t();
    return *langs;
  }
  static const Language *Default() { return def; }

  Language(Msg *_messages, const String &_family, const String &_name)
    : messages(_messages),
      family(Compare::Instance()->FindFamily(_family)),
      name(_name) {}
  Msg *Messages() const { return messages; }
  Compare::Family *Family() const { return family; }

  static const Language *Find(const String &name);
  const String &Name() const { return name; }
private:
  static const Language * const def;
  static langmap_t *langs;
  Msg *messages;
  Compare::Family *family;
  String name;
};

#endif // ACAP_LANGUAGE_H
