/* modifiers.h -*- C++ -*- modifiers used when searching
 * Rob Earhart
 * $Id: modifiers.h,v 1.5 1998/03/09 00:58:34 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_MODIFIERS_H
#define ACAP_MODIFIERS_H

#include <slist.h>
#include <pair.h>
#include "str.h"
#include "token.h"
#include "ptr.h"
#include "compare.h"

class Modifiers {
public:
  class Metadata {
  public:
    void AddMetadatum(const String &data);
    void UseDefaultMetadata(const String &attribute);
    void Reverse() { metadata.reverse(); }

    typedef slist<Token::token> metadata_t;
    metadata_t &GetMetadata() { return metadata; }

  private:
    metadata_t metadata;
  };

  void SetEnumerate();
  void SetNotify();
  void SetMakeContext(const String &context);
  void SettingReturnList();
  void AddReturn(const String &attribute, Ptr<Metadata> &metadata);
  void DoneWithReturnList();
  void SettingSortList();
  void AddSort(const String &attribute, const String &comparator);
  void DoneWithSortList();
  void SetDepth(unsigned long depth);
  void SetHardLimit(unsigned long limit);
  void SetLimit(unsigned long limit, unsigned long max);
  void SetNoInherit();
  Modifiers();

  typedef slist<pair<String, Ptr<Metadata> > > returnlist_t;
  typedef slist<pair<String, Compare::ordering_t> > sortlist_t;

  const returnlist_t &ReturnList() const { return returnlist; }
  const sortlist_t &SortList() const { return sortlist; }
  unsigned long Depth() const { return depth; }
  const String &Context() const { return makecontext; }
  bool Notify() const { return gotnotify; }
  bool Enumerate() const { return gotenumerate; }
  bool Inherit() const { return ! gotnoinherit; }

  bool GotMakeContext () const { return gotmakecontext; }
  bool GotReturn() const { return gotreturn; }
  bool GotSort() const { return gotsort; }
  bool GotDepth() const { return gotdepth; }
private:
  bool gotenumerate,
       gotnotify,
       gotmakecontext,
       gotreturn,
       gotsort,
       gotdepth,
       gothardlimit,
       gotlimit,
       gotnoinherit;
  String makecontext;
  unsigned long depth;
  unsigned long hardlimit;
  unsigned long limit, maxlimit;
  returnlist_t returnlist;
  sortlist_t sortlist;
};

#endif // ACAP_MODIFIERS_H
