/* msg.h -*- C++ -*- generic message interface
 * Rob Earhart
 * $Id: msg.h,v 1.6 1998/03/09 00:58:36 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_MSG_H
#define ACAP_MSG_H

class Msg {
public:
  typedef const char *(Msg::*Generator)();

  // Commands
  virtual const char *Authenticate() = 0;
  virtual const char *Bye() = 0;
  virtual const char *DeletedSince() = 0;
  virtual const char *FreeContext() = 0;
  virtual const char *Lang() = 0;
  virtual const char *Noop() = 0;
  virtual const char *Store() = 0;
  virtual const char *Search() = 0;
  virtual const char *UpdateContext() = 0;

  // Parsing
  virtual const char *CommandAborted() = 0;
  virtual const char *ExpectedSpace() = 0;
  virtual const char *ExpectedCRLF() = 0;
  virtual const char *ExpectedLParen() = 0;
  virtual const char *ExpectedRParen() = 0;
  virtual const char *ExpectedRParenOrSpace() = 0;
  virtual const char *ExpectedString() = 0;
  virtual const char *ExpectedStringOrNil() = 0;
  virtual const char *ExpectedQString() = 0;
  virtual const char *ExpectedNumber() = 0;
  virtual const char *ExpectedContext() = 0;
  virtual const char *CTypeInEntryAttr() = 0;
  virtual const char *NumberTooBig() = 0;
  virtual const char *UnrecWriteMetadata() = 0;
  virtual const char *ACLString() = 0;
  virtual const char *CTypeStrList() = 0;
  virtual const char *NoMultiInEntry() = 0;
  virtual const char *ExpectedValue() = 0;
  virtual const char *ExpectedStoreMetadata() = 0;
  virtual const char *NoSuchContext() = 0;
  virtual const char *NotUpdateableContext() = 0;
  virtual const char *InvalidSubdatasetSyntax() = 0;
  virtual const char *MultivaluedInherit() = 0;

  // Datasets
  virtual const char *IllegalPath() = 0;
  virtual const char *UnknownDatamechName() = 0;
  virtual const char *DatasetNoExist() = 0;
  virtual const char *TooOld() = 0;
  virtual const char *DatasetReferal() = 0;
};

#endif // ACAP_MSG_H
