/* notifycontext.h -*- C++ -*- notify context interface
 * Rob Earhart
 * $Id: notifycontext.h,v 1.8 1998/03/09 00:58:38 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_NOTIFYCONTEXT_H
#define ACAP_NOTIFYCONTEXT_H

#include "context.h"
#include "interlock.h"

class NotifyContext : public Context {
public:
  NotifyContext(const String &_name,
		Conn *_conn,
		const Timestamp::Stamp &_modtime,
		const String &_path,
		bool enumerated);
  void Noref();
  ~NotifyContext();
    
  void ScheduleUpdate();	// schedules a delta update for
				// some future time
  void Update(Timestamp *stamp); // forces an immediate update as of
				 // the supplied time; returns
				 // when the update's complete

  void DoneInitializing(entries_t &ents,
			Ptr<DatasetGroup> &group,
			Ptr<Criteria> &crit,
			Ptr<Modifiers> &mods);

  NotifyContext *NotifyCast() { return this; }

  const String &Name() const { return name; }

  void AddPathDependancy(String path);
  void AddDatasetDependancy(String path);
  static void NotifyDatasetChange(String path);
  static void NotifyPathChange(String path);

private:
  // Note: this should only be called when the NotifyPaths
  // mutex has been acquired...
  void DropDependancies();

  void Enumerate(entries_t &result,
		 DatasetGroup *group,
		 Timestamp::Stamp stamp);

  void IssueChange(unsigned old_pos, unsigned new_pos, const EntryRef *entry) const;
  void IssueAddTo(unsigned pos, const EntryRef *entry) const;
  void IssueRemoveFrom(unsigned pos, const EntryRef *entry) const;

  friend class Updater;
  String name;
  String path;
  bool needs_update;
  bool has_update_task;
  Mutex flag_mutex;
  Mutex update_mutex;
  Mutex entries_mutex;
  Ptr<Criteria> criteria;
  Ptr<Modifiers> modifiers;

  typedef hash_set<String, hash<String>, equal_to<String> > paths_t;
  paths_t paths;
};

#endif // ACAP_NOTIFYCONTEXT_H
