/* store.h -*- C++ -*- store command interface
 * Rob Earhart
 * $Id: store.h,v 1.5 1998/02/18 21:19:49 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_STORE_H
#define ACAP_STORE_H

#include <hash_map.h>
#include "command.h"
#include "str.h"
#include "token.h"
#include "storeentry.h"
#include "interlock.h"
#include "dsgroup.h"

class Store : public Command {
public:
  Store(const String &_tag, Conn *_conn)
    : Command(_tag, _conn) {}

  void Setup();			// called after the command's state's
				// been completely initialized.  It acquires
				// a timestamp and interferes with subsequent
				// searches, so it's best to call this once
				// the dispatch ordering lock's been acquired.

  void AddEntry(Ptr<StoreEntry> &ent);

private:
  void Eval();
  Msg::Generator Name() const;

  typedef hash_map<String, Ptr<StoreEntry>,
                   hash<String>, equal_to<String> > entries_t;
  entries_t entries;
  Ptr<Interlock::Version> version;
  DatasetGroup dsgroup;
};

#endif // ACAP_STORE_H
