/* storeentry.cc - storeentry implementation
 * Rob Earhart
 * $Id: storeentry.cc,v 1.2 1998/02/18 21:19:53 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include "response.h"
#include "storeentry.h"
#include "entry.h"

void StoreEntry::Attribute::AddValue(Token::token type,
				     Ptr<StoreEntry::Value> &_value) {
  Ptr<Value> *vp;
  switch (type) {
  case Token::A_ACL:
    vp = &acl;
    break;
  case Token::A_VALUE:
    vp = &value;
    break;
  case Token::A_CONTENTTYPE:
    vp = &ctype;
    break;
  default:
    throw Response::BAD("Unwriteable metadata type used in store");
  }
  if (*vp) throw Response::BAD("Duplicate metadata listed");
  *vp = _value;
}

void StoreEntry::AddAttribute(Ptr<StoreEntry::Attribute> &attr) {
  switch(attr->Special()) {
  case Token::A_ENTRY:
    if (entry_attr)
      throw Response::BAD("Attribute listed multiple times");
    entry_attr = attr;
    break;
  case Token::A_SUBDATASET:
    if (subdataset_attr)
      throw Response::BAD("Attribute listed multiple times");
    subdataset_attr = attr;
    break;
  case Token::A_DATASETINHERIT:
    if (inherit_attr)
      throw Response::BAD("Attribute listed multiple times");
    inherit_attr = attr;
    // no break - dataset.inherit also gets stored in the
    // attribute map.
  default:
    pair<attributes_t::iterator, bool> b
      = attributes.insert(attributes_t::value_type(attr->Name(), attr));
    if (! b.second)
      throw Response::BAD("Attribute listed multiple times");
  }
}

void StoreEntry::SetUnchangedSince(const String &str) {
  if (gotunchanged)
    throw Response::BAD("UnchangedSince specified multiple times for one entry");
  gotunchanged = true;
  unchangedsince = str;
}

void StoreEntry::SetNoCreate() {
  if (nocreate)
    throw Response::BAD("NoCreate specified multiple times for one entry");
  nocreate = true;
}

StoreEntry::StoreEntry(const String &_name)
  : name(_name), gotunchanged(false), nocreate(false), inherit_attr(NULL) {

    String::size_type len = name.length(), here;

    if (len < 1 || name[0] != '/')
      throw Response::BAD("Entry path must start with a '/'");

    for (here = len - 1; here >= 0; here--)
      if (name[here] == '/')
	break;

    dataset_name = String(name.octets(), here);
    entry_name = String(name.octets() + here + 1, len - here - 1);

    if (! entry_name.size())
      throw Response::BAD("Entry path must not end with a '/'");

    DEBUG("Dataset is " << dataset_name
	  << "; entry is " << entry_name);
}
